; The agent based model of labor market with job-placement agencies, heterogeneous individuals and on-the-job search.
; The model was developed as a labor market policy evaluation tool.
; The model default duration is 156 periods, the first 12 months are perceived as warm-up period, thus the evaluated model duration is 144 periods
; The model was built on a basis of labor market search theory by Dale Mortensen and Christopher Pissarides
; For more details please read the attached working paper and comments in the model code.

; TO LAUNCH THE MODEL PLEASE USE THE FOLLOWING STEPS (FOR WINDOWS):
; 1) ADD JAVA_HOME TO YOUR ENVIRONMENT VARIABLES AND SET PATH TO JAVA (E.G. /usr/local/jdk1.6.0/bin).
; 2) RUN TERMINAL AS ADMINISTRATOR
; 3) NAVIGATE TO YOUR NETLOGO FOLDER
; 4) TYPE ' JAVA -JAR NETLOGO.JAR '
; 5) OPEN THE MODEL FROM NETLOGO FILE MENU


; need r extension to import data frame to R and analyze results
; please disable if NetLogo suspends

extensions [ r ]

directed-link-breed [ ltu-links ltu-link ]
directed-link-breed [ regular-links regular-link ]
directed-link-breed [ job-links job-link ]


breed [ job-seekers job-seeker ]
breed [ firms firm ]
breed [ vacancies vacancy]
breed [ agencies agency ]


globals
  [
    elasticity
    resign-ratio
    free-vacancies
    maximum-distance
    unem-list
    otj-list
    emp-num-list
    uratio-list
    lturatio-list
    agr-jobs-list
    prod-jobs-list
    serv-jobs-list
    wage-list
    productivity-list
    ltu-productivity-list
    js-prob-list
    ltu-prob-list
    js-almp-list
    ltu-almp-list
    free-ls-list
    free-ms-list
    free-hs-list
    dur-list
    ltu-dur-list
    theta-list
    months-counter
    ]

job-seekers-own
  [
    seeker-probability
    ltu-probability
    productivity
    visited-firms
    must-go
    preferences
    skills
    unemployment-duration
    almp-duration
    employment-duration
    long-term-unemployed?
    employed?
    searching?
    bargaining?
    seeker-wage
    real-wage
    leisure
    search-units
    unemployment-value
    employment-value
    almp?
    on-the-job-search?
    on-the-job-bargaining?
    theta-js
    ]

firms-own
  [
    production-level
    all-costs
    recruiting?
    firing?
    creating-vacancies?

    ]

vacancies-own
  [
    kind
    skill-level
    recruitment-cost
    wage-offer
    vacant-value
    filled-value
    is-free?
    recruiting-duration
    producing?
    productivity-component
    production
    firm-probability
    theta-v
  ]

agencies-own
  [
    ltu-almp-places
    regular-almp-places
    ltu-enrol?
    regular-enrol?
  ]

;; ----------------------------------------------------------------------------------------------------------------
;; ----------------------------------------------------SETUP-------------------------------------------------------
;; ----------------------------------------------------------------------------------------------------------------


to setup
  clear-all
  setup-job-seekers
  setup-firms
  setup-vacancies
  setup-agencies
  general-setup
  reset-ticks
end

to setup-job-seekers
  set-default-shape job-seekers "person"
  create-job-seekers 600
  ask job-seekers
  [
      set color blue
      set size 1
      set unemployment-duration 0
      set employment-duration 0
      set almp-duration 0
      set employed? FALSE
      set searching? TRUE
      set bargaining? FALSE
      set long-term-unemployed? FALSE

      set almp? FALSE
      set on-the-job-search? FALSE
      set on-the-job-bargaining? FALSE
      set visited-firms [ ]
      set skills one-of [ 1 2 3 4 5 ]
         let random-number random-float 1
            if random-number < 0.5 [
            set preferences ["services"] ]
            if random-number < 0.8 and random-number > 0.5 [
            set preferences ["production"] ]
            if random-number < 1 and random-number > 0.8 [
            set preferences ["agricultural"] ]

          if skills <= 2 [
                set productivity random-normal 2 0.2
                set leisure random-float 0.5 ]

          if skills = 3 or skills = 4 [
                set productivity random-normal 2.5 0.2
                set leisure random-float 0.5 ]

           if skills = 5 [
                set productivity random-normal 3 0.2
                set leisure random-float 0.5 ]
      ]

end

to setup-firms
  set-default-shape firms "factory"
  create-firms 250
  ask firms
  [
      set color grey
      set size 1.5
      set recruiting? FALSE
      set firing? FALSE

    ]
end

to setup-vacancies

      let maximum 600
      let minimum 500

      create-vacancies minimum + random(maximum - minimum + 1)

; Create vacancies in the three general secors of economy. The initial distribution of the vacancies is based on emprical data,
; where about 50% of vacant jobs is in services 30% are vacancies connected with production and 20% are perceived as agricultural.
; Skill-level of vacancy is randomly drawn from a vector [1, ..., 5] and then evolves. If the firm cannot find the trading partner
; can lower skill requirements.

      ask vacancies [
       set is-free? TRUE
       set recruiting-duration 0
       set producing? FALSE
            let random-number random-float 1
            if random-number < 0.5
            [ set shape "circle 2"
              set color orange
              set size 0.7
              set kind ["services"]
              set skill-level one-of [ 1 2 3 4 5 ] ]
            if random-number < 0.8 and random-number > 0.5
            [ set shape "square 2"
              set color lime
              set size 0.6
              set kind ["production"]
              set skill-level one-of [ 1 2 3 4 5 ] ]
            if random-number < 1 and random-number > 0.8
            [ set shape "triangle 2"
              set color yellow
              set size 0.6
              set kind ["agricultural"]
              set skill-level one-of [ 1 2 3 4 5 ] ]

          if skill-level <= 2 [
            set recruitment-cost random-normal 0.6 0.2 ]
            set wage-offer minimum-wage + random-float 0.5
            set productivity-component random-normal 1 0.2

          if skill-level = 3 or skill-level = 4 [
            set recruitment-cost random-normal 0.8 0.2 ]
            set wage-offer minimum-wage + random-float 1
            set productivity-component random-normal 1.5 0.2

          if skill-level = 5 [
            set recruitment-cost random-normal 1 0.2 ]
            set wage-offer minimum-wage + random-float 1.5
            set productivity-component random-normal 2 0.2
      ]

end

to setup-agencies
  set-default-shape agencies "house"
  let coordinates [[-12 12] [-12 -12] [12 12] [12 -12]]
  foreach coordinates [ create-agencies 1 [ setxy item 0 ? item 1 ? ] ]


  ask agencies
  [
    set ltu-almp-places 0
    set regular-almp-places 0
    set color red set size 2
    set ltu-enrol? FALSE
    set regular-enrol? FALSE
  ]


end

to general-setup
  ask job-seekers
    [
      let empty-patches patches with [ not any? turtles-here and not any? agencies-on neighbors]
      if any? turtles-here [ move-to one-of empty-patches ]
    ]

  ask firms
    [ let empty-patches patches with [ not any? turtles-here and not any? firms-on neighbors and not any? agencies-on neighbors]
      move-to one-of empty-patches
    ]

  ask vacancies
  [
      let firms-patches patches with [ any? firms-here ]

         if not any? firms-here [ move-to one-of firms-patches ]

            while [count vacancies-here > 3] [ move-to one-of firms-patches ] ]


  set months-counter 0
  reset-ticks
end


;; -----------------------------------------------------------------------------------------------------------------------------------------
;; -----------------------------------------------------GO PROCEDURE------------------------------------------------------------------------
;; -----------------------------------------------------------------------------------------------------------------------------------------

to go
  if ticks >= 156 [ stop ]  ;; 1 year of warp-up period and 12 years of simulation
  initialize-search-process
  create-jobs
  draw-for-job
  update-value-functions
  start-job-search-assistance
  get-employed
  on-the-job-search
  set-wage-production
  destroy-vacancies
  update-cv
  tick
end


to initialize-search-process

; exogenous resign ratio parameter concern those who participate in ALMP support
; it is almost 5 time higher then job destruction rate (0.11)

set resign-ratio 0.05

  ask firms [
    let num-of-workers count job-seekers-here with [ employed? = TRUE ]
    let my-vacancies count vacancies-here with [is-free? = TRUE ]
    if my-vacancies > 0 and num-of-workers < my-vacancies
       [ set recruiting? TRUE ]

    if my-vacancies = 0 or num-of-workers >= my-vacancies
       [ set recruiting? FALSE ]

  ]

; Distribute search-units among job seekers. By assumption long-term unemployed job-seekeers have less maximum number of search units.
; In turn job-seekers who participate in the job search assistance program have exogenous search-units bonus with maximum number = 3.
; Job-seekers decide, if they want to start or stop ALMP program participation.

  ask job-seekers with [ searching? = TRUE ] [

    if long-term-unemployed? = TRUE and almp? = FALSE
       [ set search-units random 10

             let random-number random-float 1
               if random-number < tau-ltu and any? agencies with [ltu-enrol? = TRUE]
                [ set almp? TRUE ] ]

    if long-term-unemployed? = TRUE and almp? = TRUE
       [ set search-units random 10 + ltu-search-bonus

            let random-number random-float 1
              if random-number < resign-ratio
                 [ set almp? FALSE ] ]

    if long-term-unemployed? = FALSE and almp? = FALSE
       [ set search-units random 13

            let random-number random-float 1
              if random-number < tau-regular and any? agencies with [regular-enrol? = TRUE]
                 [ set almp? TRUE ]  ]

    if long-term-unemployed? = FALSE and almp? = TRUE
       [ set search-units random 13 + search-bonus

            let random-number random-float 1
              if random-number < resign-ratio
                 [ set almp? FALSE ] ]

       set maximum-distance max [search-units] of job-seekers
       ]

; Update the individual probability of meeting the free vacancy of given type and skill level - ABM implementation. Each job-seeker checks the number of the
; potential vacancies in the distance equal to the number of owned search units. Then each job seeker checks also the number of potential competitors in the
; distance equal to the mean number of search units of job-seeker in the neigborhood. Given this data the individual matching probability is copmputed
; based on the matching function formula: (efficiency * (v/u ^ 0.5)). The same process is undertake by firms.


    ask job-seekers with [ searching? = TRUE ] [

             let mean-of-others-search-units mean [search-units] of job-seekers in-cone maximum-distance 360

             let num-job-seekers count job-seekers in-cone mean-of-others-search-units 360 with [ searching? = TRUE and preferences = [preferences] of myself ]
             let num-on-the-job count job-seekers in-cone mean-of-others-search-units 360 with [ on-the-job-search? = TRUE and preferences = [preferences] of myself ]
             let my-free-jobs count vacancies in-cone search-units 360 with [ is-free? = TRUE and kind = [preferences] of myself ]

             if  num-job-seekers > 0 [

                set theta-js ( (my-free-jobs) / (num-job-seekers) )
                set seeker-probability ( efficiency * theta-js ^ (1 - 0.5) )
                ]

             ]

    ask vacancies with [ is-free? = TRUE ] [

             let mean-of-others-search-units mean [search-units] of job-seekers in-cone maximum-distance 360

             let num-job-seekers count job-seekers in-cone mean-of-others-search-units 360 with [ searching? = TRUE and preferences = [kind] of myself and search-units > 0]
             let num-on-the-job count job-seekers in-cone mean-of-others-search-units 360 with [ on-the-job-search? = TRUE and preferences = [kind] of myself ]
             let free-jobs count vacancies in-cone mean-of-others-search-units 360 with [ is-free? = TRUE and kind = [kind] of myself ]

             if  num-job-seekers > 0 and free-jobs > 0 [

               set theta-v ( (free-jobs) / (num-job-seekers) )

               set firm-probability ( efficiency * 1 / ( theta-v ^ 0.5) ) ]

             ]
end

; AB implementation of the job-creation condition. Firms find the potential trading partner in the distance equal to the mean number
; of search units of job seeker, who are able to make an application given turn due to sufficient search units. Then firm calculates
; maximum potential payoff from filling the new vacancy. In the third step firm calulates mean cost of maintaining the vacancy and expected
; number of periods to find the trading partner. If number-of-periods * costs <= potential-payoff the firms opens a new vacancy with random
; skill-requirements and in the random sector of economy. If the firm has no vacancies the average recruitment cost is calculated from a free
; vacancies in the maximum distance.


to create-jobs

  ask firms [

     let searchers job-seekers with [ searching? = TRUE or on-the-job-search? = TRUE]
     let my-vacancies vacancies-here

       if any? searchers and count my-vacancies < 3 and any? vacancies-here [


             let mean-of-others-search-units mean [search-units] of job-seekers in-cone maximum-distance 360

             let num-job-seekers count job-seekers in-cone mean-of-others-search-units 360 with [ searching? = TRUE]
             let num-on-the-job count job-seekers in-cone mean-of-others-search-units 360 with [ on-the-job-search? = TRUE ]
             let other-firms count firms in-cone mean-of-others-search-units 360

             if  num-job-seekers > 0 and other-firms > 0 [

               let t ( (other-firms) / (num-job-seekers) )
               let matching-prob ( efficiency * 1 / ( t ^ 0.5) )

                 let my-searchers job-seekers in-cone maximum-distance 360 with [ searching? = TRUE ]
                 let my-vacs vacancies-here
                 let p max [productivity] of my-searchers
                 let c  mean [recruitment-cost] of my-vacs
                 let w  mean [wage-offer] of my-vacs
                 let x max [productivity-component] of my-vacs

                 let expected-profit (p * x - w)
                 let expected-cost c * 1 / matching-prob


                   if expected-profit >= expected-cost [

                     let my-patch patch-here
                      ask my-patch [ sprout-vacancies 1 [ set is-free? TRUE set recruiting-duration 0 set producing? FALSE
                         let random-number random-float 1
                            if random-number < 0.5
                             [ set shape "circle 2"
                               set color orange
                               set size 0.7
                               set kind ["services"]
                               set skill-level one-of [ 1 2 3 4 5 ] ]
                            if random-number < 0.8 and random-number > 0.5
                             [ set shape "square 2"
                               set color lime
                               set size 0.6
                               set kind ["production"]
                               set skill-level one-of [ 1 2 3 4 5 ] ]
                            if random-number < 1 and random-number > 0.8
                             [ set shape "triangle 2"
                               set color yellow
                               set size 0.6
                               set kind ["agricultural"]
                               set skill-level one-of [ 1 2 3 4 5 ] ]
                            if skill-level <= 2 [
                               set recruitment-cost random-normal 0.6 0.2 ]
                               set wage-offer minimum-wage + random-float 0.5
                               set productivity-component random-normal 1 0.2
                            if skill-level = 3 or skill-level = 4 [
                               set recruitment-cost random-normal 0.8 0.2 ]
                               set wage-offer minimum-wage + random-float 1
                               set productivity-component random-normal 1.5 0.2
                            if skill-level = 5 [
                               set recruitment-cost random-normal 1 0.2 ]
                               set wage-offer minimum-wage + random-float 1.5
                               set productivity-component random-normal 2 0.2 ]  ]
                 ]
             ]
      ]

  ]

 ask firms [

     let searchers job-seekers with [ searching? = TRUE or on-the-job-search? = TRUE ]
      if not any? vacancies-here and any? searchers [

        let mean-of-others-search-units mean [search-units] of job-seekers in-cone maximum-distance 360

             let num-job-seekers count job-seekers in-cone mean-of-others-search-units 360 with [ searching? = TRUE]
             let num-on-the-job count job-seekers in-cone mean-of-others-search-units 360 with [ on-the-job-search? = TRUE ]
             let other-firms count firms in-cone mean-of-others-search-units 360

             if  num-job-seekers > 0 and other-firms > 0 [

               let t ( (other-firms) / (num-job-seekers) )
               let matching-prob ( efficiency * 1 / ( t ^ 0.5) )

                 let my-searchers job-seekers in-cone mean-of-others-search-units 360 with [ searching? = TRUE]

                 let p max [productivity] of my-searchers
                 let c  mean [recruitment-cost] of vacancies in-cone maximum-distance 360
                 let w  mean [wage-offer] of vacancies in-cone maximum-distance  360
                 let x max [productivity-component] of vacancies in-cone maximum-distance 360


                 if p * x - w >= c * 1 / matching-prob [

                    let my-patch patch-here
                      ask my-patch [ sprout-vacancies 1 [ set is-free? TRUE set recruiting-duration 0 set producing? FALSE
                         let random-number random-float 1
                            if random-number < 0.5
                             [ set shape "circle 2"
                               set color orange
                               set size 0.7
                               set kind ["services"]
                               set skill-level one-of [ 1 2 3 4 5 ] ]
                            if random-number < 0.8 and random-number > 0.5
                             [ set shape "square 2"
                               set color lime
                               set size 0.6
                               set kind ["production"]
                               set skill-level one-of [ 1 2 3 4 5 ] ]
                            if random-number < 1 and random-number > 0.8
                             [ set shape "triangle 2"
                               set color yellow
                               set size 0.6
                               set kind ["agricultural"]
                               set skill-level one-of [ 1 2 3 4 5 ] ]
                            if skill-level <= 2 [
                               set recruitment-cost random-normal 0.6 0.2 ]
                               set wage-offer minimum-wage + random-float 0.5
                               set productivity-component random-normal 1 0.2
                            if skill-level = 3 or skill-level = 4 [
                               set recruitment-cost random-normal 0.8 0.2 ]
                               set wage-offer minimum-wage + random-float 1
                               set productivity-component random-normal 1.5 0.2
                            if skill-level = 5 [
                               set recruitment-cost random-normal 1 0.2 ]
                               set wage-offer minimum-wage + random-float 1.5
                               set productivity-component random-normal 2 0.2 ]  ]
                 ]
             ]
      ]

  ]

end


; AB implementation of the search and match algoithm. Each job-seeker can search for a job as long as the number of search units is > 0. Job seeker
; has limited information about firms that are recruitng workers and see only preferable free vacancies in the maximum distance. Then he calulates the amount
; of the search units which are demanded to send an aplication. In the third step job seeker chooses the free vacancy which can be achieved with the
; minimum number of search units and move to the target. If he aplicates with no effect and still has search units, he can continue the search
; process until search units = 0. If he applicates succesfully, the NASH solution is aplied to compute the individual wage.

to draw-for-job

  ask job-seekers with [ searching? = TRUE  ] [

    while [ search-units > 0 ] [

    let remaining-firms other vacancies-on patches in-cone maximum-distance 360 with [ not member? self [visited-firms] of myself ]
       let free-remaining-firms remaining-firms with [is-free? = TRUE]

       if not any? free-remaining-firms [set visited-firms [ ] set search-units 0 ]

       if any? free-remaining-firms

          [ face min-one-of free-remaining-firms [distance myself]
            forward 1
            set search-units (search-units - 1)

             if any? vacancies-here

              [ set visited-firms (patch-set patch-here visited-firms)

                let my-jobs vacancies-here with [ is-free? = TRUE
                and kind = [preferences] of myself and skill-level <= [skills] of myself ]


                if any? my-jobs

               [

                 create-job-link-to max-one-of my-jobs [ skill-level ]

                 set bargaining? TRUE
                 set searching? FALSE
                 set search-units 0

                 ask my-jobs [ if any? my-in-job-links [ set is-free? FALSE ] ]

                 ]


                ]
    ]
  ]

]

ask job-seekers [ if search-units < 0 [set search-units 0]]

end


;;---------------------------------------------------------------------------------------------------------------------------------------------------
;;--------------------------------------------------------JOB AGENCY MODULE--------------------------------------------------------------------------
;;---------------------------------------------------------------------------------------------------------------------------------------------------

; Each of unemployed can participate in the job-search assistance program (ALMP), which imply potential rise in the search units as well as the posibility
; of utilize job adverts gathered by agencies. The maximum number of program participants is set to 40% of population of unemployed. Inflow to ALMP
; is exogenous paramerter in range 0-30. Each of participants can resign from ALMP for some external reasons with probability set to 0.05.


to start-job-search-assistance


; Agencies calculates the number of free ALMP places, which is based on the population of unemployed and set
; to maximum 30% of unemployed

ask agencies [

  let ltu-num count job-seekers with [ searching? = TRUE and long-term-unemployed? = TRUE ]
  let reg-num count job-seekers with [ searching? = TRUE and long-term-unemployed? = FALSE ]
  let age-num count agencies

  set ltu-almp-places (0.4 * (ltu-num / age-num))
  set regular-almp-places (0.4 * (reg-num / age-num ))

  let ltu-participants count my-in-ltu-links
  let reg-participants count my-in-regular-links

  ifelse reg-participants >= regular-almp-places

      [ set regular-enrol? FALSE ]
      [ set regular-enrol? TRUE ]

  ifelse ltu-participants < ltu-almp-places
     [ set ltu-enrol? TRUE ]
     [ set ltu-enrol? FALSE ]


]


; Job seekers who participates in ALMP program besides gaining extra search units can utilize
; job adverts which are available in the office. Each turn unemployed who participate in ALMP decides
; if they want to visit the nearest agency and sample from job adverts acquired by the agency

ask job-seekers [

  if searching? = TRUE and long-term-unemployed? = FALSE and almp? = TRUE

  [

  let targets agencies with [ regular-enrol? = TRUE ]

   if any? targets [

      let my-agency min-one-of targets [distance myself]
      let places [ regular-almp-places ] of my-agency
      create-regular-link-to my-agency

      let participants count [my-in-regular-links] of my-agency
      let must-die (participants - places)
      if must-die > 0

          [ set almp? FALSE
            let empty-patches patches with [ not any? turtles-here ]
            move-to min-one-of empty-patches [distance myself]
            ask my-out-regular-links [die] ]

      if must-die <= 0

       [ let random-number random-float 1
           if random-number < agency-adds
            [ move-to my-agency ] ]
        ]


     ]


  if searching? = TRUE and long-term-unemployed? = TRUE and almp? = TRUE

  [

  let targets agencies with [ ltu-enrol? = TRUE ]

  if any? targets [

      let my-agency min-one-of targets [distance myself]
      let places [ ltu-almp-places ] of my-agency
      create-ltu-link-to my-agency

      let participants count [my-in-ltu-links] of my-agency
      let must-die (participants - places)


      if must-die > 0

          [ set almp? FALSE
            let empty-patches patches with [ not any? turtles-here ]
            move-to min-one-of empty-patches [distance myself]
            ask my-out-ltu-links [die] ]

      if must-die <= 0

       [ let random-number random-float 1
           if random-number < ltu-agency-adds
            [ move-to my-agency ] ]
         ]


     ]

]

; Job seekers have three list of job adverts to sample from. The choice of the list depend on
; the inidividual preferences. Algoritm assumes that offers available in agencies are more fitted
; to the individual skills of job seekers due to job councelors and other staff who works
; with unemployed and help him to decide where to applicates for a job

ask job-seekers with [ searching? = TRUE and almp? = TRUE and any? agencies-here ] [

         if preferences = ["services"]

           [

             let services-job vacancies in-cone maximum-distance 360 with [ kind = ["services"] and is-free? = TRUE ]
                 if any? services-job with [ skill-level <= [skills] of myself ]
                   [ let target max-one-of services-job [skill-level]
                     create-job-link-to target
                     move-to target
                     ask target [ set is-free? FALSE ]
                     set bargaining? TRUE set searching? FALSE]

           ]

         if preferences = ["production"]

            [

             let production-jobs vacancies in-cone maximum-distance 360 with [ kind = ["production"] and is-free? = TRUE ]
                 if any? production-jobs with [ skill-level <= [skills] of myself ]
                   [ let target max-one-of production-jobs [skill-level]
                     create-job-link-to target
                     move-to target
                     ask target [ set is-free? FALSE ]
                     set bargaining? TRUE set searching? FALSE]

            ]

         if preferences = ["agricultural"]

            [

             let agri-jobs vacancies in-cone maximum-distance 360 with [ kind = ["agricultural"] and is-free? = TRUE ]
                 if any? agri-jobs with [ skill-level <= [skills] of myself ]
                   [ let target max-one-of agri-jobs [skill-level]
                     create-job-link-to target
                     move-to target
                     ask target [ set is-free? FALSE ]
                     set bargaining? TRUE set searching? FALSE]

            ]
]


end


;----------------------------------------------------------------------------------------------------------------------------------------------------
;-------------------------------------------------------------BELLMAN EQUATIONS UPDATE---------------------------------------------------------------
;----------------------------------------------------------------------------------------------------------------------------------------------------

; ABM implementation of Bellman equations. Agents compere present and future payoffs from the search process and make the best desisions
; in order to maximize present income. Than NASH solution is aplied for agents who faces the bargaining problem.

to update-value-functions

; Job seeker who meets proper vacancy calculetes present payoffs from accepting job proposal and future payoffs
; from continue the job search and find the better vacancy next period. The distribution of future job offers is
; limited to the maximum number of search units. If future payoffs are less then the payoffs from current job,
; job seeker stops the search process and accept the job

  ask job-seekers [
    if bargaining? = TRUE [
      let my-future-jobs other vacancies in-cone maximum-distance 360 with [ is-free? = TRUE and kind = [preferences] of myself and skill-level <= [skills] of myself ]
      let my-curent-job out-job-link-neighbors
      let my-job-skills first [skill-level] of my-curent-job
      let current-payoff first [wage-offer] of my-curent-job
      if any? my-future-jobs
        [ let future-payoff max [wage-offer] of my-future-jobs
          set unemployment-value (benefits + leisure + (seeker-probability * future-payoff) )

           ifelse my-job-skills < skills
               [ set employment-value (current-payoff - shock * (  current-payoff - unemployment-value ) ) ]
               [ set employment-value (current-payoff - shock * (  current-payoff - unemployment-value ) + (seeker-probability * future-payoff)) ] ]

      if not any? my-future-jobs  [ set unemployment-value (benefits + leisure)
                                    set employment-value current-payoff - shock * (  current-payoff - unemployment-value ) ]

    ]
  ]


; Vacancy who meets the proper candidate for the job calculates present payoffs from hiring the candidate and future payoffs
; from continue the recruiting process and find the better candidate next period. The distribution of future candidate is
; determined by the maximum distance in a given period. If future payoffs are lees then the payoffs from job seeker who currently
; visits the vacancy, firm decides to stop the recruitment process and hire current job seeker.

  ask vacancies [
    if any? my-in-job-links
    [
      let my-future-seekers job-seekers in-cone maximum-distance 360 with [searching? = TRUE and preferences = [kind] of myself and skills >= [skill-level] of myself ]
      let my-current-seeker in-job-link-neighbors
      let my-seeker-skills [skills] of my-current-seeker
      let my-current-payoff first [productivity] of my-current-seeker * productivity-component

      if any? my-future-seekers
         [ let future-payoff max [productivity] of my-future-seekers * productivity-component
           set vacant-value ( - recruitment-cost + firm-probability * future-payoff)

           ifelse my-seeker-skills = skill-level
            [ set filled-value ( my-current-payoff - wage-offer - shock * ( filled-value - vacant-value )) ]
            [ set filled-value ( my-current-payoff - wage-offer - shock * ( filled-value - vacant-value ) + firm-probability * future-payoff ) ] ]

      if not any? my-future-seekers [ set vacant-value (- recruitment-cost)
                                      set filled-value ( my-current-payoff - wage-offer - shock * ( filled-value - vacant-value )) ]


      ]


    if not any? my-in-job-links [set is-free? TRUE]

  ]

end


; NASH bargaining solution: The condiiton to get employed is that the [job seeker - vacancy] match must be profitable for both agents.
; If and only if the employment value > unemployment value and filled value > vacant value the job seeker starts the work.

to get-employed

  ask job-seekers [

    if bargaining? = TRUE and unemployment-value > employment-value
    [ ask my-out-job-links [die] set bargaining? FALSE set searching? TRUE ]

     if bargaining? = TRUE and unemployment-value <= employment-value [
       let my-vacancy out-job-link-neighbors
       let V first [vacant-value] of my-vacancy
       let F first [filled-value] of my-vacancy
         ifelse V > F
          [ ask my-out-job-links [die] set bargaining? FALSE set searching? TRUE ]
          [ set employed? TRUE set bargaining? FALSE ]

     if employed? = TRUE and any? my-out-regular-links [ ask my-out-regular-links [die] ]
     if employed? = TRUE and any? my-out-ltu-links [ ask my-out-ltu-links [die] ]


     ]
  ]

  ask vacancies [ if not any? my-in-job-links [ set is-free? TRUE ] ]

end

;; -------------------------------------------------------On-the-job search procedure----------------------------------------------------------------------
;; Workers who works below their qualifications are able to seek on-the-job with a given probability. If they find the proper vacancy
;; their value of employment is updated and worker continue producing on the new job with new wage. Note that due to random distribution
;; of productivity p and stochastic compononet of job production x the vacancy with higher skill demand can offer the lower wage.
;; In that case worker do not change the job.

to on-the-job-search

 ask job-seekers [
   if employed? = TRUE and any? my-out-job-links
    [
       let me self
         ask me [ let my-job out-job-link-neighbors

                    let z [skill-level] of my-job
                    if first z < skills [ set on-the-job-search? TRUE ] ]
        ]
 ]


 ask job-seekers [
   if on-the-job-search? = TRUE and any? my-out-job-links
   [
      let search-on-the-job? random-float 1
       if search-on-the-job? < 0.1 [

           let my-job out-job-link-neighbors
           let my-job-skill first [skill-level] of my-job
           let targets vacancies in-cone maximum-distance 360 with [ is-free? = TRUE and skill-level > my-job-skill
                                                       and skill-level <= [skills] of myself ]

            if any? targets [

              let future-job one-of targets
              let F [ filled-value ] of future-job
              let V [ vacant-value ] of future-job

               if F > V and employment-value > unemployment-value [ ask my-out-job-links [ die ]
                                                                    move-to future-job
                                                                    create-job-link-to future-job
                                                                    set color white
                                                                    ask future-job [set is-free? FALSE ] ]
            ]


      ]

   ]

 ]

 ask vacancies [ if not any? my-in-job-links [ set is-free? TRUE ] ]

end

;----------------------------------------------------------------------------------------------------------------------------------------------------
;------------------------------------------------------ UPDATE PRODUCTION AND CALCULATE REAL WAGE ---------------------------------------------------
;----------------------------------------------------------------------------------------------------------------------------------------------------
; Production is the result of the individual productivity p, job productivity component x and AR(1) process which drives produictivity changes over time
; Because exogenous growth rate, the individual productivity rise over time.

to set-wage-production
  ask vacancies [
    if any? my-in-job-links [
        let my-worker in-job-link-neighbors
        let p0 first [productivity] of my-worker
        let low-limit -0.10 - 0.8 * growth-rate
        let hi-limit 0.10 + growth-rate
        let stochastic low-limit + random-float ( hi-limit - low-limit)

             set production ((p0 + p0 * stochastic) * productivity-component)
             ask my-worker [set productivity p0 + p0 * stochastic]
    ]
 ]

  ask job-seekers [
    if employed? = TRUE [

        let my-job out-job-link-neighbors
        let px first [production] of my-job
        let V first [vacant-value] of my-job
        set real-wage ( (1 - beta) * unemployment-value + beta * px ) ]

        ]

  ask firms
     [ if any? vacancies-here with [ is-free? = FALSE ] [

         if any? job-seekers-here with [ employed? = TRUE] [

         let prod sum [production] of vacancies-here with [ is-free? = FALSE ]
         let workers job-seekers-here with [ employed? = TRUE ]

          set production-level ( ( prod / count workers ) ) ]

  ]

       if not any? job-seekers-here with [ employed? = TRUE ] [ set production-level 0 ]
     ]

end


;;---------------------------------------------------------------------------------------------------------------------------------------------------
;;--------------------------------------------------------JOB DESTRUCTION-----------------------------------------------------------
;;---------------------------------------------------------------------------------------------------------------------------------------------------

;; Job destruction is the result of random shocks which hit the jobs. If the firm is hit by the shock must fire the worker and close the job.
;; The shock frequency is calibrated with Latin Hypercube Sampling technique. If firm unsuccesfully recruits the workers can lower the skill
;; demand with given probability. If the recruitment exceeds 6 months, the firm can destroy vacant position.


to destroy-vacancies


ask job-seekers with [ employed? = TRUE and employment-duration > 6 ]

    [

      let random-number random-float 1

      if random-number < shock [

        let me self
         ask me [

            let destroyed-vacancy out-job-link-neighbors

             set employed? FALSE
             set searching? TRUE
             set long-term-unemployed? FALSE
             set unemployment-duration 0
             set employment-duration 0
             set on-the-job-search? FALSE
             set color red
             if productivity < 0.8 [ set productivity 0.8 ]
                ask my-out-links [die]
                ask destroyed-vacancy [die]

              ]

           ]

 ]

ask job-seekers with [ employed? = TRUE and employment-duration < 6 ]

    [

      let random-number random-float 1

      if random-number < shock [

        let me self
         ask me [

            let destroyed-vacancy out-job-link-neighbors

             set employed? FALSE
             set searching? TRUE
             set unemployment-duration 0
             set employment-duration 0
             set on-the-job-search? FALSE
             set color yellow
             if productivity < 0.8 [ set productivity 0.8 ]

                ask my-out-links [die]
                ask destroyed-vacancy [die]

             ]

           ]

 ]

ask vacancies with [is-free? = TRUE] [
                   set recruitment-cost (recruitment-cost * recruiting-duration)
                   set recruiting-duration recruiting-duration + 1
                   let random-variable random-float 1
                     if random-variable < 0.05 and skill-level > 1
                       [ set skill-level (skill-level - 1)
                          if skill-level <= 2 [
                               set wage-offer minimum-wage + random-float 0.5
                               set productivity-component random-normal 1 0.2 ]

                            if skill-level = 3 or skill-level = 4 [
                               set wage-offer minimum-wage + random-float 1
                               set productivity-component random-normal 1.5 0.2 ]
                       ]

                      if recruiting-duration > 6
                        [
                          let me self
                              ask me [ let destroy? random-float 1
                                if destroy? < 0.1 [ ask me [die] ]
                              ]
                        ]


  ]

ask vacancies with [is-free? = FALSE] [
  set recruiting-duration 0 ]

end


;; update the state-dependent information in CVs: individual productivity, duration of employment and unemployment
;; If job seeker is unemployed his individual productivity falls every period with 70% of growth rate dynamic. Job
;; seeker, while unemployed also struggles with skills depreciacion. In turn while employed can increase skill level.
;; If job seeker cannot find the job have an opportunity to change the preferences. This decision is based on the labor
;; market recognition in a maximum distance.

to update-cv

set months-counter (months-counter + 1)

  ask job-seekers [
    if searching?
       [ set unemployment-duration (unemployment-duration + 1)


         if unemployment-duration > 12 [ set color green
                                         set long-term-unemployed? TRUE ]

         if productivity > 0.7
            [ set productivity (productivity - (0.5 * growth-rate * productivity) )
              ]
            ]


    if employed?
       [ set employment-duration (employment-duration + 1)
         set unemployment-duration 0
         if skills < 5 [
           let random-number random-float 1
           let skill-upgrade-probability 0.01

           if random-number < skill-upgrade-probability [ set skills skills + 1 ] ]

       ]


    if long-term-unemployed? and skills > 1 [
       let random-number random-float 1
       let depreciacion-probability 0.01

          if random-number < depreciacion-probability [ set skills skills - 1]

    ]

;    if long-term-unemployed? = FALSE and skills > 1 [
;       let random-number random-float 1
;       let depreciacion-probability 0.01

;          if random-number < depreciacion-probability [ set skills skills - 1]

;    ]

  ]

ask job-seekers with [ employed? = FALSE ] [

        if months-counter mod 6 = 0 [

          let serv count vacancies in-cone maximum-distance 360 with [ kind = ["services"] and is-free? = TRUE  ]
          let prod count vacancies in-cone maximum-distance 360 with [ kind = ["production"] and is-free? = TRUE ]
          let agri count vacancies in-cone maximum-distance 360 with [ kind = ["agricultural"] and is-free? = TRUE ]


            if serv > prod and serv > agri [ set preferences ["services"]]
            if prod > serv and prod > agri [ set preferences ["production"]]
            if agri > serv and agri > prod [ set preferences ["agricultural"]]

                ]  ]

end

;; here starts the analytical section, economic inidicators are reported
;; data are put in data frame with r extension
;; plots, calibration and sensitivity analysis are performed with R programming language
;; see additional scripts for details

to-report unemployment-rate
  let unemployed count job-seekers with [ searching? = TRUE ]
  let population count job-seekers
  report ( unemployed / population ) * 100
end

to-report unem
   report count job-seekers with [ searching? = TRUE ]
end

to-report otj
  report count job-seekers with [on-the-job-search? = TRUE]
end

to-report emp-num
  report count job-seekers with [employed? = TRUE]
end

to-report ltu-rate
  let ltu count job-seekers with [ long-term-unemployed? = TRUE and searching? = TRUE ]
  let unemployed count job-seekers with [ searching? = TRUE and employed? = FALSE ]
  ifelse unemployed > 0
    [ report ( ltu / unemployed ) * 100 ]
    [ report 0 ]
end

to-report theta
  let js count job-seekers with [searching? = TRUE]
  let ojs count job-seekers with [on-the-job-search? = TRUE]
  let v count vacancies with [is-free? = TRUE]

  report (v / (js + ojs) )
end

to-report ltu-prob
  let ltus job-seekers with [long-term-unemployed? = TRUE and searching? = TRUE]
  ifelse any? ltus
  [ report mean [seeker-probability] of ltus ]
  [ report 0 ]
end


to-report ltu-almp-prob
  let ltus job-seekers with [long-term-unemployed? = TRUE and searching? = TRUE and almp? = TRUE]
  ifelse any? ltus
  [ report mean [seeker-probability] of ltus ]
  [ report 0 ]
end

 to-report js-prob
  let js job-seekers with [long-term-unemployed? = FALSE and searching? = TRUE]
  ifelse any? js
  [ report mean [seeker-probability] of js]
  [ report 0 ]
end

to-report js-almp-prob
  let js job-seekers with [long-term-unemployed? = FALSE and searching? = TRUE and almp? = TRUE]
  ifelse any? js
  [ report mean [seeker-probability] of js]
  [ report 0 ]
end


to-report prd
  let js job-seekers with [long-term-unemployed? = FALSE and employed? = TRUE]
  ifelse any? js
  [ report mean [productivity] of js]
  [ report 0 ]
end

to-report ltu-prd
  let js job-seekers with [long-term-unemployed? = TRUE and employed? = TRUE]
  ifelse any? js
  [ report mean [productivity] of js]
  [ report 0 ]
end

to-report wag
  let js job-seekers with [employed? = TRUE]
  ifelse any? js
  [ report mean [real-wage] of js]
  [ report 0 ]
end


to-report agr-jobs
  let vacs vacancies with [kind = ["agricultural"]]
  ifelse any? vacs
  [ report count vacs ]
  [ report 0 ]
end

to-report serv-jobs
  let vacs vacancies with [kind = ["services"]]
  ifelse any? vacs
  [ report count vacs ]
  [ report 0 ]
end

to-report prod-jobs
  let vacs vacancies with [kind = ["production"]]
  ifelse any? vacs
  [ report count vacs ]
  [ report 0 ]
end

to-report ls-vacs
  let vacs vacancies with [skill-level = 1 ]
  ifelse any? vacs
  [ report count vacs ]
  [ report 0 ]
end

to-report ms-vacs
  let vacs vacancies with [skill-level < 4 and skill-level >= 2 ]
  ifelse any? vacs
  [ report count vacs ]
  [ report 0 ]
end

to-report hs-vacs
  let hvacs vacancies with [skill-level >= 4]
  ifelse any? hvacs
  [ report count hvacs ]
  [ report 0 ]
end

to-report dur
  let js job-seekers with [searching? = TRUE and long-term-unemployed? = FALSE ]
  ifelse any? js
  [report mean [unemployment-duration] of js]
  [report 0]
end

to-report ltu-dur
  let js job-seekers with [searching? = TRUE and long-term-unemployed? = TRUE ]
  ifelse any? js
  [report mean [unemployment-duration] of js]
  [report 0]
end

to-report mismatch
  let jobs count vacancies with [is-free? = FALSE]
  let ot count job-seekers with [on-the-job-search? = TRUE]
  ifelse jobs > 0
  [report ot / jobs]
  [report ot ]
end

to-report time
  report months-counter
end

to store-data
  if ticks > 12
  [
  set unem-list lput unem unem-list
  set otj-list lput otj otj-list
  set emp-num-list lput emp-num emp-num-list
  set uratio-list lput unemployment-rate uratio-list
  set lturatio-list lput ltu-rate lturatio-list
  set agr-jobs-list lput agr-jobs agr-jobs-list
  set prod-jobs-list lput prod-jobs prod-jobs-list
  set serv-jobs-list lput serv-jobs serv-jobs-list
  set wage-list lput wag wage-list
  set productivity-list lput prd productivity-list
  set ltu-productivity-list lput ltu-prd ltu-productivity-list
  set js-prob-list lput js-prob js-prob-list
  set ltu-prob-list lput ltu-prob ltu-prob-list
  set js-almp-list lput js-almp-prob js-almp-list
  set ltu-almp-list lput ltu-almp-prob ltu-almp-list
  set free-ls-list lput ls-vacs free-ls-list
  set free-ms-list lput ms-vacs free-ms-list
  set free-hs-list lput hs-vacs free-hs-list
  set theta-list lput theta theta-list
  set dur-list lput dur dur-list
  set ltu-dur-list lput ltu-dur ltu-dur-list
  set months-counter months-counter + 1
  ]
end


to gen-dataframe

  ( r:putdataframe "df1" "unem" unem-list "otj" otj-list "emp" emp-num-list
    "urate" uratio-list "lturate" lturatio-list "agr" agr-jobs-list "prod" prod-jobs-list
    "serv" serv-jobs-list "w" wage-list "p" productivity-list "ltup" ltu-productivity-list
    "prob" js-prob-list "ltu_prob" ltu-prob-list "almp_prob" js-almp-list
    "almp_pltu" ltu-almp-list "lsv" free-ls-list "msv" free-ms-list "hsv" free-hs-list "theta" theta-list
    "dur" dur-list "ltudur" ltu-dur-list)

  r:eval "save(df1, file='C:/Users/wozni_000/Desktop/df1.Rda')"

end


@#$#@#$#@
GRAPHICS-WINDOW
288
27
904
664
20
20
14.8033
1
10
1
1
1
0
0
0
1
-20
20
-20
20
0
0
1
ticks
30.0

BUTTON
107
13
184
46
setup
setup
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

BUTTON
10
13
96
46
go
go
T
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

SLIDER
7
52
99
85
beta
beta
0.4
0.6
0.472
0.01
1
NIL
HORIZONTAL

MONITOR
220
109
271
154
theta
theta
17
1
11

SLIDER
6
89
98
122
shock
shock
0
0.04
0.015
0.001
1
NIL
HORIZONTAL

SLIDER
105
225
197
258
tau-ltu
tau-ltu
0
0.4
0
0.05
1
NIL
HORIZONTAL

SLIDER
104
192
196
225
tau-regular
tau-regular
0
0.4
0
0.01
1
NIL
HORIZONTAL

MONITOR
219
11
276
56
u-rate
unemployment-rate
17
1
11

SLIDER
4
191
98
224
benefits
benefits
0.3
1.2
0.84
0.01
1
NIL
HORIZONTAL

PLOT
6
661
253
781
rates
NIL
NIL
0.0
10.0
0.0
10.0
true
true
"" ""
PENS
"u-rate" 1.0 0 -13345367 true "" "plot unemployment-rate"
"ltu-rate" 1.0 0 -14439633 true "" "plot ltu-rate"

MONITOR
221
61
278
106
NIL
ltu-rate
17
1
11

MONITOR
220
157
270
202
jobs
count vacancies
17
1
11

SLIDER
107
51
199
84
search-bonus
search-bonus
0
5
0
1
1
NIL
HORIZONTAL

MONITOR
220
204
270
249
unem
count job-seekers with [searching? = TRUE]
17
1
11

SLIDER
103
89
196
122
ltu-search-bonus
ltu-search-bonus
0
5
0
1
1
NIL
HORIZONTAL

SLIDER
104
124
198
157
agency-adds
agency-adds
0
0.4
0
0.1
1
NIL
HORIZONTAL

SLIDER
105
158
199
191
ltu-agency-adds
ltu-agency-adds
0
0.4
0
0.05
1
NIL
HORIZONTAL

SLIDER
6
156
98
189
minimum-wage
minimum-wage
0.5
1.5
1.29
0.1
1
NIL
HORIZONTAL

PLOT
7
538
250
658
probabilities
NIL
NIL
0.0
10.0
0.0
0.9
true
true
"" ""
PENS
"unemployed" 1.0 0 -16777216 true "" "plot js-prob"
"ltu" 1.0 0 -14439633 true "" "plot ltu-prob"

SLIDER
6
123
98
156
efficiency
efficiency
0.1
0.3
0.256
0.01
1
NIL
HORIZONTAL

PLOT
6
418
248
538
job seekers and vacancies
NIL
NIL
0.0
10.0
0.0
10.0
true
true
"" ""
PENS
"employed" 1.0 0 -16777216 true "" "plot count job-seekers with [employed? = TRUE]"
"otj" 1.0 0 -15040220 true "" "plot count job-seekers with [on-the-job-search? = TRUE]"
"vacant" 1.0 0 -2674135 true "" "plot count vacancies with [is-free? = TRUE]"

PLOT
6
297
250
417
productivity
NIL
NIL
0.0
10.0
0.0
4.0
true
true
"" ""
PENS
"prod" 1.0 0 -16777216 true "" "plot prd"
"ltu-prod" 1.0 0 -2674135 true "" "plot ltu-prd"
"wage" 1.0 0 -12087248 true "" "plot wag"

PLOT
260
666
511
786
jobs
NIL
NIL
0.0
10.0
0.0
10.0
true
true
"" ""
PENS
"agricultural" 1.0 0 -16777216 true "" "plot agr-jobs"
"production" 1.0 0 -5298144 true "" "plot prod-jobs"
"services" 1.0 0 -12087248 true "" "plot serv-jobs"

SLIDER
3
224
98
257
growth-rate
growth-rate
0
0.05
0.03
0.01
1
NIL
HORIZONTAL

BUTTON
25
262
182
295
dataframe
gen-dataframe
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

PLOT
513
666
737
786
duration
NIL
NIL
0.0
10.0
0.0
10.0
true
true
"" ""
PENS
"dur" 1.0 0 -16777216 true "" "plot dur"
"ltu-dur" 1.0 0 -15040220 true "" "plot ltu-dur"

@#$#@#$#@
## WHAT IS IT?

(a general understanding of what the model is trying to show or explain)

## HOW IT WORKS

(what rules the agents use to create the overall behavior of the model)

## HOW TO USE IT

(how to use the model, including a description of each of the items in the Interface tab)

## THINGS TO NOTICE

(suggested things for the user to notice while running the model)

## THINGS TO TRY

(suggested things for the user to try to do (move sliders, switches, etc.) with the model)

## EXTENDING THE MODEL

(suggested things to add or change in the Code tab to make the model more complicated, detailed, accurate, etc.)

## NETLOGO FEATURES

(interesting or unusual features of NetLogo that the model uses, particularly in the Code tab; or where workarounds were needed for missing features)

## RELATED MODELS

(models in the NetLogo Models Library and elsewhere which are of related interest)

## CREDITS AND REFERENCES

(a reference to the model's URL on the web if it has one, as well as any other necessary credits, citations, and links)
@#$#@#$#@
default
true
0
Polygon -7500403 true true 150 5 40 250 150 205 260 250

airplane
true
0
Polygon -7500403 true true 150 0 135 15 120 60 120 105 15 165 15 195 120 180 135 240 105 270 120 285 150 270 180 285 210 270 165 240 180 180 285 195 285 165 180 105 180 60 165 15

arrow
true
0
Polygon -7500403 true true 150 0 0 150 105 150 105 293 195 293 195 150 300 150

box
false
0
Polygon -7500403 true true 150 285 285 225 285 75 150 135
Polygon -7500403 true true 150 135 15 75 150 15 285 75
Polygon -7500403 true true 15 75 15 225 150 285 150 135
Line -16777216 false 150 285 150 135
Line -16777216 false 150 135 15 75
Line -16777216 false 150 135 285 75

bug
true
0
Circle -7500403 true true 96 182 108
Circle -7500403 true true 110 127 80
Circle -7500403 true true 110 75 80
Line -7500403 true 150 100 80 30
Line -7500403 true 150 100 220 30

butterfly
true
0
Polygon -7500403 true true 150 165 209 199 225 225 225 255 195 270 165 255 150 240
Polygon -7500403 true true 150 165 89 198 75 225 75 255 105 270 135 255 150 240
Polygon -7500403 true true 139 148 100 105 55 90 25 90 10 105 10 135 25 180 40 195 85 194 139 163
Polygon -7500403 true true 162 150 200 105 245 90 275 90 290 105 290 135 275 180 260 195 215 195 162 165
Polygon -16777216 true false 150 255 135 225 120 150 135 120 150 105 165 120 180 150 165 225
Circle -16777216 true false 135 90 30
Line -16777216 false 150 105 195 60
Line -16777216 false 150 105 105 60

car
false
0
Polygon -7500403 true true 300 180 279 164 261 144 240 135 226 132 213 106 203 84 185 63 159 50 135 50 75 60 0 150 0 165 0 225 300 225 300 180
Circle -16777216 true false 180 180 90
Circle -16777216 true false 30 180 90
Polygon -16777216 true false 162 80 132 78 134 135 209 135 194 105 189 96 180 89
Circle -7500403 true true 47 195 58
Circle -7500403 true true 195 195 58

circle
false
0
Circle -7500403 true true 0 0 300

circle 2
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240

cow
false
0
Polygon -7500403 true true 200 193 197 249 179 249 177 196 166 187 140 189 93 191 78 179 72 211 49 209 48 181 37 149 25 120 25 89 45 72 103 84 179 75 198 76 252 64 272 81 293 103 285 121 255 121 242 118 224 167
Polygon -7500403 true true 73 210 86 251 62 249 48 208
Polygon -7500403 true true 25 114 16 195 9 204 23 213 25 200 39 123

cylinder
false
0
Circle -7500403 true true 0 0 300

dot
false
0
Circle -7500403 true true 90 90 120

face happy
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 255 90 239 62 213 47 191 67 179 90 203 109 218 150 225 192 218 210 203 227 181 251 194 236 217 212 240

face neutral
false
0
Circle -7500403 true true 8 7 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Rectangle -16777216 true false 60 195 240 225

face sad
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 168 90 184 62 210 47 232 67 244 90 220 109 205 150 198 192 205 210 220 227 242 251 229 236 206 212 183

factory
false
0
Rectangle -7500403 true true 76 194 285 270
Rectangle -7500403 true true 36 95 59 231
Rectangle -16777216 true false 90 210 270 240
Line -7500403 true 90 195 90 255
Line -7500403 true 120 195 120 255
Line -7500403 true 150 195 150 240
Line -7500403 true 180 195 180 255
Line -7500403 true 210 210 210 240
Line -7500403 true 240 210 240 240
Line -7500403 true 90 225 270 225
Circle -1 true false 37 73 32
Circle -1 true false 55 38 54
Circle -1 true false 96 21 42
Circle -1 true false 105 40 32
Circle -1 true false 129 19 42
Rectangle -7500403 true true 14 228 78 270

fish
false
0
Polygon -1 true false 44 131 21 87 15 86 0 120 15 150 0 180 13 214 20 212 45 166
Polygon -1 true false 135 195 119 235 95 218 76 210 46 204 60 165
Polygon -1 true false 75 45 83 77 71 103 86 114 166 78 135 60
Polygon -7500403 true true 30 136 151 77 226 81 280 119 292 146 292 160 287 170 270 195 195 210 151 212 30 166
Circle -16777216 true false 215 106 30

flag
false
0
Rectangle -7500403 true true 60 15 75 300
Polygon -7500403 true true 90 150 270 90 90 30
Line -7500403 true 75 135 90 135
Line -7500403 true 75 45 90 45

flower
false
0
Polygon -10899396 true false 135 120 165 165 180 210 180 240 150 300 165 300 195 240 195 195 165 135
Circle -7500403 true true 85 132 38
Circle -7500403 true true 130 147 38
Circle -7500403 true true 192 85 38
Circle -7500403 true true 85 40 38
Circle -7500403 true true 177 40 38
Circle -7500403 true true 177 132 38
Circle -7500403 true true 70 85 38
Circle -7500403 true true 130 25 38
Circle -7500403 true true 96 51 108
Circle -16777216 true false 113 68 74
Polygon -10899396 true false 189 233 219 188 249 173 279 188 234 218
Polygon -10899396 true false 180 255 150 210 105 210 75 240 135 240

house
false
0
Rectangle -7500403 true true 45 120 255 285
Rectangle -16777216 true false 120 210 180 285
Polygon -7500403 true true 15 120 150 15 285 120
Line -16777216 false 30 120 270 120

leaf
false
0
Polygon -7500403 true true 150 210 135 195 120 210 60 210 30 195 60 180 60 165 15 135 30 120 15 105 40 104 45 90 60 90 90 105 105 120 120 120 105 60 120 60 135 30 150 15 165 30 180 60 195 60 180 120 195 120 210 105 240 90 255 90 263 104 285 105 270 120 285 135 240 165 240 180 270 195 240 210 180 210 165 195
Polygon -7500403 true true 135 195 135 240 120 255 105 255 105 285 135 285 165 240 165 195

line
true
0
Line -7500403 true 150 0 150 300

line half
true
0
Line -7500403 true 150 0 150 150

pentagon
false
0
Polygon -7500403 true true 150 15 15 120 60 285 240 285 285 120

person
false
0
Circle -7500403 true true 110 5 80
Polygon -7500403 true true 105 90 120 195 90 285 105 300 135 300 150 225 165 300 195 300 210 285 180 195 195 90
Rectangle -7500403 true true 127 79 172 94
Polygon -7500403 true true 195 90 240 150 225 180 165 105
Polygon -7500403 true true 105 90 60 150 75 180 135 105

plant
false
0
Rectangle -7500403 true true 135 90 165 300
Polygon -7500403 true true 135 255 90 210 45 195 75 255 135 285
Polygon -7500403 true true 165 255 210 210 255 195 225 255 165 285
Polygon -7500403 true true 135 180 90 135 45 120 75 180 135 210
Polygon -7500403 true true 165 180 165 210 225 180 255 120 210 135
Polygon -7500403 true true 135 105 90 60 45 45 75 105 135 135
Polygon -7500403 true true 165 105 165 135 225 105 255 45 210 60
Polygon -7500403 true true 135 90 120 45 150 15 180 45 165 90

sheep
false
15
Circle -1 true true 203 65 88
Circle -1 true true 70 65 162
Circle -1 true true 150 105 120
Polygon -7500403 true false 218 120 240 165 255 165 278 120
Circle -7500403 true false 214 72 67
Rectangle -1 true true 164 223 179 298
Polygon -1 true true 45 285 30 285 30 240 15 195 45 210
Circle -1 true true 3 83 150
Rectangle -1 true true 65 221 80 296
Polygon -1 true true 195 285 210 285 210 240 240 210 195 210
Polygon -7500403 true false 276 85 285 105 302 99 294 83
Polygon -7500403 true false 219 85 210 105 193 99 201 83

square
false
0
Rectangle -7500403 true true 30 30 270 270

square 2
false
0
Rectangle -7500403 true true 30 30 270 270
Rectangle -16777216 true false 60 60 240 240

star
false
0
Polygon -7500403 true true 151 1 185 108 298 108 207 175 242 282 151 216 59 282 94 175 3 108 116 108

target
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240
Circle -7500403 true true 60 60 180
Circle -16777216 true false 90 90 120
Circle -7500403 true true 120 120 60

tree
false
0
Circle -7500403 true true 118 3 94
Rectangle -6459832 true false 120 195 180 300
Circle -7500403 true true 65 21 108
Circle -7500403 true true 116 41 127
Circle -7500403 true true 45 90 120
Circle -7500403 true true 104 74 152

triangle
false
0
Polygon -7500403 true true 150 30 15 255 285 255

triangle 2
false
0
Polygon -7500403 true true 150 30 15 255 285 255
Polygon -16777216 true false 151 99 225 223 75 224

truck
false
0
Rectangle -7500403 true true 4 45 195 187
Polygon -7500403 true true 296 193 296 150 259 134 244 104 208 104 207 194
Rectangle -1 true false 195 60 195 105
Polygon -16777216 true false 238 112 252 141 219 141 218 112
Circle -16777216 true false 234 174 42
Rectangle -7500403 true true 181 185 214 194
Circle -16777216 true false 144 174 42
Circle -16777216 true false 24 174 42
Circle -7500403 false true 24 174 42
Circle -7500403 false true 144 174 42
Circle -7500403 false true 234 174 42

turtle
true
0
Polygon -10899396 true false 215 204 240 233 246 254 228 266 215 252 193 210
Polygon -10899396 true false 195 90 225 75 245 75 260 89 269 108 261 124 240 105 225 105 210 105
Polygon -10899396 true false 105 90 75 75 55 75 40 89 31 108 39 124 60 105 75 105 90 105
Polygon -10899396 true false 132 85 134 64 107 51 108 17 150 2 192 18 192 52 169 65 172 87
Polygon -10899396 true false 85 204 60 233 54 254 72 266 85 252 107 210
Polygon -7500403 true true 119 75 179 75 209 101 224 135 220 225 175 261 128 261 81 224 74 135 88 99

wheel
false
0
Circle -7500403 true true 3 3 294
Circle -16777216 true false 30 30 240
Line -7500403 true 150 285 150 15
Line -7500403 true 15 150 285 150
Circle -7500403 true true 120 120 60
Line -7500403 true 216 40 79 269
Line -7500403 true 40 84 269 221
Line -7500403 true 40 216 269 79
Line -7500403 true 84 40 221 269

wolf
false
0
Polygon -16777216 true false 253 133 245 131 245 133
Polygon -7500403 true true 2 194 13 197 30 191 38 193 38 205 20 226 20 257 27 265 38 266 40 260 31 253 31 230 60 206 68 198 75 209 66 228 65 243 82 261 84 268 100 267 103 261 77 239 79 231 100 207 98 196 119 201 143 202 160 195 166 210 172 213 173 238 167 251 160 248 154 265 169 264 178 247 186 240 198 260 200 271 217 271 219 262 207 258 195 230 192 198 210 184 227 164 242 144 259 145 284 151 277 141 293 140 299 134 297 127 273 119 270 105
Polygon -7500403 true true -1 195 14 180 36 166 40 153 53 140 82 131 134 133 159 126 188 115 227 108 236 102 238 98 268 86 269 92 281 87 269 103 269 113

x
false
0
Polygon -7500403 true true 270 75 225 30 30 225 75 270
Polygon -7500403 true true 30 75 75 30 270 225 225 270

@#$#@#$#@
NetLogo 5.2.0
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
default
0.0
-0.2 0 0.0 1.0
0.0 1 1.0 0.0
0.2 0 0.0 1.0
link direction
true
0
Line -7500403 true 150 150 90 180
Line -7500403 true 150 150 210 180

@#$#@#$#@
0
@#$#@#$#@
